{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWAboutDialog.pas                                      }
{ Short description  : Windows shellabout function wrapper                     }
{ Author             : Carsten Butzek                                          }
{ E-Mail             : cabutzek@muenster.de                                    }
{ Internet           :                                                         }
{ Version            : 1.0                                                     }
{ Date last modified : 12.04.2000                                              }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWAboutDialog                                           }
{ Classes          :                                                           }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWAboutDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ShellApi, DsgnIntf;

const
      ComponentVersion='1.0';
      ComponentAuthor='Carsten Butzek';
      ComponentEMail='cabutzek@muenster.de';
      ComponentHomePage='http://www.delphifreeware.de';

type
  TDFWAboutDialog = class(TComponent)
  private
    FAppName: TCaption;
    FCaption: TCaption;
    FExtraInfo: TCaption;
    FIcon: TIcon;
    procedure SetIcon(value: TIcon);
  protected
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
  public
    function Execute: boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property AppName: TCaption read FAppName write FAppName;
    property Caption: TCaption read FCaption write FCaption;
    property ExtraInfo: TCaption read FExtraInfo write FExtraInfo;
    property Icon: TIcon read FIcon write SetIcon;
  end;

implementation

procedure TDFWAboutDialog.SetIcon(value: TIcon);
begin
  FIcon.Assign(value);
end;

constructor TDFWAboutDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIcon := TIcon.Create;
end;

function TDFWAboutDialog.Execute: boolean;
var
  TempStr: string;
begin
  if (Caption = '') and (AppName = '') then
     TempStr := Application.Title else
   if (Caption = '') or (AppName = '') then
     TempStr := Caption+AppName
   else
     TempStr := Caption+'#'+AppName;

  if Icon.Empty then
    Result := Boolean(ShellAbout((Owner as TForm).Handle, PChar(TempStr), PChar(ExtraInfo),
                      Application.Icon.Handle))
  else
    Result := Boolean(ShellAbout((Owner as TForm).Handle, PChar(TempStr), PChar(ExtraInfo),
                      FIcon.Handle));
end;

destructor TDFWAboutDialog.Destroy;
begin
  FIcon.Free;
  inherited Destroy;
end;

function TDFWAboutDialog.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWAboutDialog.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWAboutDialog.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWAboutDialog.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
