{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWAnalogMeter.pas                                      }
{ Short description  :                                                         }
{ Author             : Hannes Breuer                                           }
{ E-Mail             : hannes.breuer@talknet.de                                    }
{ Internet           : http://www.talknet.de/~hannes.breuer/                   }
{ Version            : 2.03                                                    }
{ Date last modified : 17.1.2000                                               }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   : 2.0  First release version, never mind the private      }
{                           version's history...                               }
{                      2.01 Split DrawFace into DrawOuterFace and              }
{                           DrawInnerFace for speed.                           }
{                           Cleaned up uses-clause                             }
{                      2.02 Derived it from TCustomControl and moved WMSize to }
{                           protected part, now WM_SIZE arrives correctly      }
{                      2.03 Changed the comments such that the F1help-compiler }
{                           HLP-file for it. Added that helpfile, of course.   }
{                           See www.geocities.com/SiliconValley/Vista/5524/f1help.html }
{                           for the latest version of this really excellent    }
{                           tool.                                              }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWAnalogMeter                                           }
{ Classes          :                                                           }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWAnalogMeter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls{ExtCtrls};

const
      ComponentVersion='2.03';
      ComponentAuthor='Hannes Breuer';
      ComponentEMail='hannes.breuer@talknet.de';
      ComponentHomePage='http://www.talknet.de/~hannes.breuer/';

type
  TDFWAnalogMeter = class(TCustomControl)
  private
    //purely internal stuff:
    fBitmap : TBitMap;
    fZeroAngle     : Single;  //depends on AngularRange, counted Clockwise from straight down, in Degrees
    fCentrePoint   : TPoint;  //gets shifted according to size and Angular range
    fOuterHalfAxes,
    fInnerHalfAxes : TPoint;  //dimensions of ellipses
    fFrameWidth : LongInt;    //adjusted according to size
    fTxtHeight     : LongInt; //gets set in Paint according to Font
    fOldValue : LongInt;      //needed to XOR over old display
    //events:
    FonResize : TNotifyEvent;
    FonChange : TNotifyEvent;
    FonRiseAboveHigh : TNotifyEvent;
    FonFallBelowHigh : TNotifyEvent;
    FonRiseAboveLow : TNotifyEvent;
    FonFallBelowLow : TNotifyEvent;
    fEnableZoneEvents : Boolean;   //allow switching them off wholesale
    //variables for properties:
    fCaption : String;
    fMin, fMax, fValue : LongInt;  //just like TSpinEdit
    fAngularRange : LongInt;       //portion of circle to be used for display, 10..360 (degrees)
    fTickCount : LongInt;          //how many should be shown (including first & last)
    fLowZone,
    fHighZone : Single;            //real values, Set/Get work with percentage
    fShowFrame,
    fShowTicks,
    fShowValue : Boolean;
    fTickColor,
    fHighZoneColor,
    fLowZoneColor,
    fOKZoneColor : TColor;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    Procedure SetCaption(s : String);
    Procedure SetMin(m : LongInt);
    Procedure SetMax(m : LongInt);
    Procedure SetValue(v : LongInt);
    Procedure SetAngularRange(r : LongInt);
    Procedure SetTickCount(t : LongInt);
    Procedure SetLowZone(percent : Byte);
    Function GetLowZone : Byte;
    Procedure SetHighZone(percent : Byte);
    Function GetHighZone : Byte;
    Procedure SetShowValue(b : Boolean);
    Procedure SetHighZoneColor(c : TColor);
    Procedure SetLowZoneColor(c : TColor);
    Procedure SetOKZoneColor(c : TColor);
    Procedure SetTickColor(c : TColor);
    Procedure SetShowTicks(b : Boolean);
    Procedure SetShowFrame(b : Boolean);
    Procedure SetFont(F : TFont);
    Function GetFont : TFont;
    //drawing routines:
    Function AngleOf(v : Single) : Single; //calcs angle corresponding to value v
    Procedure OptimizeSizes; //according to dimensions and angular range
    Procedure ClearCanvas;
    Procedure DrawFrame;
    Procedure DrawRegions;
    Procedure ClearInnerFace;
    Procedure DrawTicks;
    Procedure DrawCaption;
    Procedure DrawPointer;
    Procedure DrawValue;
    Procedure DeleteOldValue;
    Procedure DrawOuterFace; //onto Bitmap in background
    Procedure DrawInnerFace; //onto Bitmap in background
  protected
    Procedure WMSize(var Message : TWMSize); message WM_SIZE;
    Procedure Paint; override; //BitBlt Background-Bitmap onto canvas
  public
    //The usual Constructor. Also creates the background bitmap which gets
    //BitBlt onto the Canvas
    constructor Create(AOwner: TComponent); override;
    //Frees the background bitmap, too
    destructor Destroy; override;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    //The caption appears inside the face above the pointer's origin.
    //See also Font, ShowValue
    property Caption : String read fCaption write SetCaption;
    //Minimum value, just like TSpinEdit's Min-property
    //See also: Max, Value
    property Min : LongInt read fMin write SetMin default 0;
    //Maximum value, just like TSpinEdit's Max-property
    //See also: Min, Value
    property Max : LongInt read fMax write SetMax default 100;
    //Current Value, just like TSpinEdit's Value
    //See also Min, Max, ShowValue
    property Value : LongInt read fValue write SetValue default 0;
    //Section of circle used for the meter.
    //Range: 10..360 (degrees)
    property AngularRange : LongInt read fAngularRange write SetAngularRange default 270;
    //No of ticks to be displayed on the face.
    //Make this an odd number to get a tick at 50%
    //Range: 0..(Max-Min), less than 2 gets set to 0.
    //See also: TickCount, TickColor
    property TickCount : LongInt read fTickCount write SetTickCount default 7;
    //Three zones can be defined: LowZone, HighZone and the zone inbetween (OKZone).
    //The low zone is defined as all values between Min and LowZone.
    //Definitions are in % of (Max-Min). If you do not want a low-zone, set this to 0.
    //If you do not want the OK-zone, set HighZone to the same percentage as LowZone.
    //Its colour is defined by LowZoneColor.
    //Its transition-events are: From low to OK: onRiseAboveLow
    //                           From OK to low: onFallBelowLow
    //See also: EnableZoneEvents
    property LowZone : Byte read GetLowZone write SetLowZone default 10;
    //Three zones can be defined: LowZone, HighZone and the zone inbetween(OKZone).
    //The high zone is defined as all values between HighZone and Max.
    //Definitions are in % of (Max-Min). If you do not want a high-zone, set this to 100.
    //If you do not want the OK-zone, set HighZone to the same percentage as LowZone.
    //Its colour is defined by HighZoneColor.
    //Its transition-events are: From OK to High: onRiseAboveHigh
    //                           From High to OK: onFallBelowHigh
    //See also: EnableZoneEvents
    property HighZone : Byte read GetHighZone write SetHighZone default 90;
    //Color of the HighZone on the meter's face.
    //See also: LowZoneColor, OKZoneColor
    property HighZoneColor : TColor read fHighZoneColor write SetHighZoneColor default clRed;
    //Color of the LowZone on the meter's face.
    //See also: HighZoneColor, OKZoneColor
    property LowZoneColor  : TColor read fLowZoneColor write SetLowZoneColor default clYellow;
    //Color of the zone inbetween LowZone and HighZone on the meter's face.
    //See also: LowZoneColor, HighZoneColor
    property OKZoneColor   : TColor read fOKZoneColor write SetOKZoneColor default cl3DLight;
    //Colour of the ticks and of the pointer on the meter's face
    //See also: TickCount, ShowTicks
    property TickColor : TColor read fTickColor write SetTickColor default clBlack;
    //Defines whether the ticks get displayed
    //See also: TickCount, TickColor
    property ShowTicks : Boolean read fShowTicks write SetShowTicks default True;
    //Defines whether a frame gets drawn around the meter's face
    property ShowFrame : Boolean read fShowFrame write SetShowFrame default True;
    //Defines whether the current Value gets displayed (below the pointer's origin).
    //Alternatively, you can set this to False and use onChange to update a TLabel
    //anywhere on your form.
    //See also: Font
    property ShowValue : Boolean read fShowValue write SetShowValue default True;
    //The face's dimensions get recalculated when the size changes.
    //The space needed for the Value at the bottom of the meter remains the same!
    //Most of the calculations are done in OptimizeSizes (private function)
    property onResize : TNotifyEvent read fOnResize write FonResize;
    //This event gets triggered everytime the Value changes.
    //See also: onRiseAboveHigh, onFallBelowHigh, onRiseAboveLow, onFallBelowLow
    property onChange : TNotifyEvent read FonChange write FonChange;
    //Defines whether the events onRiseAboveHigh, onFallBelowHigh, onRiseAboveLow, onFallBelowLow
    //get triggered.
    //See also: LowZone, HighZone
    property EnableZoneEvents : Boolean read fEnableZoneEvents write fEnableZoneEvents default True;
    //This event gets triggered if the value changes from below HighZone to above it.
    //If the value changes from below LowZone to above HighZone, the event onRiseAboveLow
    //gets suppressed.
    //See also: EnableZoneEvents, onFallBelowHigh, onFallBelowLow
    property onRiseAboveHigh : TNotifyEvent
        read FonRiseAboveHigh
       write FonRiseAboveHigh;
    //This event gets triggered if the value changes from above HighZone to below it.
    //If the value changes from above HighZone to below LowZone, this event
    //gets suppressed and only onFallBelowLow gets triggered.
    //See also: EnableZoneEvents, onRiseAboveHigh, onRiseAboveLow
    property onFallBelowHigh : TNotifyEvent
        read FonFallBelowHigh
       write FonFallBelowHigh;
    //This event gets triggered if the value changes from below LowZone to above it.
    //If the value changes from below LowZone to above HighZone, this event
    //gets suppressed and only onRiseAboveHigh gets triggered.
    //See also: EnableZoneEvents, onFallBelowHigh, onFallBelowLow
    property onRiseAboveLow : TNotifyEvent
        read FonRiseAboveLow
       write FonRiseAboveLow;
    //This event gets triggered if the value changes from above LowZone to below it.
    //If the value changes from above HighZone to below LowZone, the event onFallBelowHigh
    //gets suppressed.
    //See also: EnableZoneEvents, onRiseAboveHigh, onRiseAboveLow
    property onFallBelowLow : TNotifyEvent
        read FonFallBelowLow
       write FonFallBelowLow;
    //Purely inherited property
    property Align;
    //Inherited property
    //Defines the basic colour of the meter's face
    property Color;
    //Defines the fontkind for Value and Caption
    property Font : TFont read GetFont write SetFont;
    //Purely inherited property
    //See also: Color
    property ParentColor;
    //Purely inherited property
    //See also : Font
    property ParentFont;
    //Purely inherited property
    property ParentShowHint;
    //Purely inherited property
    property PopupMenu;
    //Purely inherited property
    property ShowHint;
    //Purely inherited property
    property Visible;
    //Purely inherited event
    property OnClick;
    //Purely inherited event
    property OnDblClick;
    //Purely inherited event
    property OnDragDrop;
    //Purely inherited event
    property OnDragOver;
    //Purely inherited event
    property OnEndDrag;
    //Purely inherited event
    property OnMouseDown;
    //Purely inherited event
    property OnMouseMove;
    //Purely inherited event
    property OnMouseUp;
    //Purely inherited event
    property OnStartDrag;
(*    property OnEnter;    NOPE, defined in TControl, not visible to me
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;    *)
  end;


implementation


{-------------------some necessary utils---------------------------------------}
{------------------------------------------------------------------------------}
Function Radius(Theta : Single; HalfAxes : TPoint) : Single;
         //calculates length of 'radius' in ellipse defined by the half-axes at
         //angle Theta (0 is straight down, clockwise, in degrees)
Var C, S : Single;  //Cos / Sin
    w, h : LongInt; //half-axes
Begin
  C := Cos(Theta*Pi/180);
  S := Sin(Theta*Pi/180);
  w := HalfAxes.X;
  h := HalfAxes.Y;
  If (w <= 0) or (h <= 0) then begin
    Result := 0;
    Exit;
  end;
  Result := w*h/Sqrt(h*h*S*S + w*w*C*C);
End;

{------------------------------------------------------------------------------}
Function XinEllipse(Theta : Single; HalfAxes : TPoint) : LongInt;
         //calculates X-posn of point on ellipse at angle Theta
Var R : Single;
Begin
  R := Radius(Theta,Halfaxes);
  Result := Round(-R*Sin(Theta*Pi/180)); //theta counted from straight down, clockwise
End;

{------------------------------------------------------------------------------}
Function YinEllipse(Theta : Single; HalfAxes : TPoint) : LongInt;
         //calculates Y-posn of point on ellipse at angle Theta
Var R : Single;
Begin
  R := Radius(Theta,Halfaxes);
  Result := Round(R*Cos(Theta*Pi/180)); //theta counted from straight down, clockwise
End;

{------------------------------------------------------------------------------}
Function OptimalHalfHeight(AvailableHeight : LongInt; Theta : Single; Width : LongInt) : LongInt;
         //calculates optimal ellipse half-height if only part of the ellipse needs to fit
         //into the AvailableHeight. Theta is the ZeroAngle
Var UsedHeight : Single;
    H : LongInt; //half-height, found by trial as the equation is not
                 // analytically solvable       // by me ;)
    C, S, T : Single; //Cos, Sin and a temp. variable for speed
Begin
  If Theta >= 90 then begin //centre MUST be shown
    Result := AvailableHeight;
    Exit;
  end;
  S := Sin(Theta*Pi/180);
  S := Sqr(S);  //only the sqr is needed later on
  C := Cos(Theta*Pi/180);
  C := Width*C/2; //only the product is needed later on
  T := Sqr(C);
  //OK, start at H = 1/2 of available and Inc(H) it until it gets too big:
  H := AvailableHeight div 2;
  UsedHeight := H + H*C / Sqrt(Sqr(H)*S + T); //basically H +Radius*Cos(Theta), just sped up
  While UsedHeight < AvailableHeight do begin
    Inc(H);
    UsedHeight := H + H*C / Sqrt(Sqr(H)*S + T);
  end;
  Dec(H); //remember that it was Inc'ed once too often
  Result := H;
End;

{-------------------------TDFWAnalogMeter-----------------------------------------}
{------------------------------------------------------------------------------}
Constructor TDFWAnalogMeter.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  fBitmap := TBitmap.Create;
  //defaults:
  Width  := 121;
  Height := 117;
  fCaption := 'mV';
  fMin   := 0;
  fMax   := 100;
  fValue := 0;
  fOldValue := fValue;
  fAngularRange  := 270;
  fTickCount     := 11;
  fLowZone   := 10;
  fHighZone  := 90;
  fShowValue := True;
  //internal stuff:
  fZeroAngle := 45;
  fTxtHeight := 8;
  fHighZoneColor := clRed;
  fLowZoneColor  := clYellow;
  fOKZoneColor   := cl3DLight;
  fTickColor     := clBlack;
  fShowTicks     := True;
  fShowFrame     := True;
  fEnableZoneEvents := True;
End;

{------------------------------------------------------------------------------}
Destructor TDFWAnalogMeter.Destroy;
Begin
  fBitmap.Free;
  inherited Destroy;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.WMSize(var Message : TWMSize);
Begin
  If Height < 37 then Height := 37; //anything else is nonsense
  If Width  < 52 then Width  := 52;
  If Assigned(FonResize) then FonResize(Self);
  OptimizeSizes;
  fBitmap.Width  := Width;
  fBitmap.Height := Height;
  DrawOuterFace;
  DrawInnerFace;
//  Paint;
  inherited;
End;

{------------------------------------------------------------------------------}
Function TDFWAnalogMeter.GetFont : TFont;
Begin
  Result := fBitmap.Canvas.Font;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetFont(f : TFont);
Begin
  If fBitmap.Canvas.Font = f then Exit;
  fBitmap.Canvas.Font := f;
  fTxtHeight := fBitmap.Canvas.TextHeight('S');
  If fShowValue or (Trim(fCaption) <> '') then begin
    DrawOuterFace;
    DrawInnerFace;
    Paint;
  end;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetCaption(s : String);
Begin
  s := Trim(s);
  If fCaption = s then Exit;
  fCaption := s;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetMin(m : LongInt);
Var OldLow, OldHigh : Longint;
Begin
  If m > fMax then m := fMax;    //force consistency
  If fMin = m then Exit;         //no action needed
  OldLow := GetLowZone;          //in %!
  OldHigh := GetHighZone;
  fMin := m;
  If fValue < m then Value := m; //force consistency
  SetLowZone(OldLow);            //converts internally
  SetHighZone(OldHigh);
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetMax(m : LongInt);
Var OldLow, OldHigh : Longint;
Begin
  If m < fMin then m := fMin;    //force consistency
  If fMax = m then Exit;         //no action needed
  OldLow := GetLowZone;          //in %!
  OldHigh := GetHighZone;
  fMax := m;
  If fValue > m then Value := m; //force consistency
  SetLowZone(OldLow);            //converts internally
  SetHighZone(OldHigh);
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetValue(v : LongInt);
Var TriggerHi, TriggerLo : Boolean;
Begin
  If v < fMin then v := fMin;
  If v > fMax then v := fMax;
  If v = fValue then Exit;
  fOldValue := fValue;
  fValue := v;
  If Assigned(FonChange) then FonChange(Self);
  //redraw:
  DrawInnerFace;
  Paint;
  //call events if necessary:
  If not fEnableZoneEvents then Exit;
  TriggerHi := Assigned(FonRiseAboveHigh) and (fOldValue < fHighZone) and (fValue >= fHighZone);
  TriggerLo := Assigned(FonFallBelowLow) and (fOldValue >= fLowZone) and (fValue < fLowZone);
  If TriggerLo then FonFallBelowLow(Self);
  If TriggerHi then FonRiseAboveHigh(Self);
  If not (TriggerHi or TriggerLo) then begin//only trigger one the two events if we move through 2 zones:
    If Assigned(FonFallBelowHigh) and (fOldValue >= fHighZone) and (fValue < fHighZone)
      then FonFallBelowHigh(Self);
    If Assigned(FonRiseAboveLow) and (fOldValue < fLowZone) and (fValue >= fLowZone)
      then FonRiseAboveLow(Self);
  end;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetAngularRange(r : LongInt);
Begin
  If r < 10 then r := 10;         //hard limits
  If r > 360 then r := 360;
  If r = fAngularRange then Exit; //no action needed
  fAngularRange := r;
  fZeroAngle := (360-AngularRange)/2;
  OptimizeSizes;
  //redraw:
  DrawOuterFace;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetTickCount(t : LongInt);
Begin
  If t > (fMax-fMin) then t := Fmax - fMin;
  If t < 2 then t := 0;                     //hard limits
  If t = fTickCount then Exit;              //no action needed
  fTickCount := t;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetLowZone(percent : Byte);
Var fOldLow : Single;
Begin
  If percent < 0   then percent := 0;   //hard limits
  If percent > 100 then percent := 100;
  If percent = LowZone then Exit; //no action needed
  If percent > HighZone then HighZone := percent; //ensure integrity
  fOldLow := fLowZone;
  fLowZone := Min + (Max-Min)*percent/100;
  DrawOuterFace;
  DrawInnerFace;
  Paint;
  //call events if necessary:
  If not fEnableZoneEvents then Exit;
  If Assigned(FonFallBelowLow) and (fValue >= fOldLow) and (fValue < fLowZone)
    then FonFallBelowLow(Self);
  If Assigned(FonRiseAboveLow) and (fValue < fOldLow) and (fValue >= fLowZone)
    then FonRiseAboveLow(Self);
End;

{------------------------------------------------------------------------------}
Function TDFWAnalogMeter.GetLowZone : Byte;
Begin
  If Max = Min
    then Result := 0
    else Result := Round(100*(fLowZone - Min)/(Max - Min));
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetHighZone(percent : Byte);
Var fOldHigh : Single;
Begin
  If percent < 0   then percent := 0;   //hard limits
  If percent > 100 then percent := 100;
  If percent = HighZone then Exit; //no action needed
  If percent < LowZone then LowZone := percent; //ensure integrity
  fOldHigh := fHighZone;
  fHighZone := Min + (Max-Min)*percent/100;
  DrawOuterFace;
  DrawInnerFace;
  Paint;
  //call events if necessary:
  If not fEnableZoneEvents then Exit;
  If Assigned(FonFallBelowHigh) and (fValue >= fOldHigh) and (fValue < fHighZone)
    then FonFallBelowHigh(Self);
  If Assigned(FonRiseAboveHigh) and (fValue < fOldHigh) and (fValue >= fHighZone)
    then FonRiseAboveHigh(Self);
End;

{------------------------------------------------------------------------------}
Function TDFWAnalogMeter.GetHighZone : Byte;
Begin
  If Max = Min
    then Result := 0
    else Result := Round(100*(fHighZone - Min)/(Max - Min));
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetShowValue(b : Boolean);
Begin
  If b = fShowValue then Exit;
  fShowValue := b;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetHighZoneColor(c : TColor);
Begin
  If fHighZoneColor = c then Exit;
  fHighZoneColor := c;
  DrawOuterFace;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetLowZoneColor(c : TColor);
Begin
  If fLowZoneColor = c then Exit;
  fLowZoneColor := c;
  DrawOuterFace;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetOKZoneColor(c : TColor);
Begin
  If fOKZoneColor = c then Exit;
  fOKZoneColor := c;
  DrawOuterFace;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetTickColor(c : TColor);
Begin
  If fTickColor = c then Exit;
  fTickColor := c;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetShowTicks(b : Boolean);
Begin
  If fShowTicks = b then Exit;
  fShowTicks := b;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.SetShowFrame(b : Boolean);
Begin
  If fShowFrame = b then Exit;
  fShowFrame := b;
  DrawOuterFace;
  DrawInnerFace;
  Paint;
End;

{------------------------------------------------------------------------------}
{--------------- drawing routines: all on fBitmap in background ---------------}
{------------------------------------------------------------------------------}
Function TDFWAnalogMeter.AngleOf(v : Single) : Single;
Begin
  Result := fZeroAngle + fAngularRange*(v - fMin)/(fMax-fMin);
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.OptimizeSizes;
var TH : LongInt; //space for text at the bottom
Begin
  TH := fTxtHeight + 2;
  fFrameWidth := (Width + Height) div 40;
  fOuterHalfAxes.Y := OptimalHalfHeight(Height - 2*fFrameWidth - TH, fZeroAngle, Width - 2*fFrameWidth);
  fCentrePoint.Y := fFrameWidth + fOuterHalfAxes.Y;
  fCentrePoint.X := Width Div 2;
  fOuterHalfAxes.X := fCentrePoint.X-fFrameWidth;
  fInnerHalfAxes := Point(fOuterHalfAxes.X - ((Width+Height)div 25),
                          fOuterHalfAxes.Y - ((Width+Height)div 25));
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.ClearCanvas;
Begin
  with fBitmap.Canvas do begin
    Brush.Style := bsClear;
    Brush.Color := Self.Color;
    FillRect(ClientRect);
  end;
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DrawFrame;
Begin
  with fBitmap.Canvas do begin
    Pen.Color := clBtnShadow;
    Pen.Width := 1;
    MoveTo(0,Height);
    LineTo(0,0);
    LineTo(Width-1,0);
    Pen.Color := clBtnHighLight;
    LineTo(Width-1,Height-1);
    LineTo(0,Height-1);
  end;
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DrawRegions;
Var Xzero, Yzero,
    Xlow, Ylow,
    XHi, YHi : LongInt;
    LowAngle, HiAngle : Single;
Begin
  //3 arcs for the 3 zones:
  LowAngle := AngleOf(fLowZone);
  HiAngle  := AngleOf(fHighZone);
  Xzero := XInEllipse(fZeroAngle,fOuterHalfAxes);
  Yzero := YInEllipse(fZeroAngle,fOuterHalfAxes);
  XLow  := XInEllipse(LowAngle,fOuterHalfAxes);
  YLow  := YInEllipse(LowAngle,fOuterHalfAxes);
  XHi   := XInEllipse(HiAngle,fOuterHalfAxes);
  YHi   := YInEllipse(HiAngle,fOuterHalfAxes);
   //fill zones with colours:
  with fBitmap.Canvas do begin
    Pen.Width := 2;
    If Color = clBtnShadow then Pen.Color := cl3DLight
                           else Pen.Color := clBtnShadow;
    If fLowZone > fMin then begin
      Brush.Color := fLowZoneColor;
      Pie(fCentrePoint.X-fOuterHalfAxes.X, fCentrePoint.Y-fOuterHalfAxes.Y,
          fCentrePoint.X+fOuterHalfAxes.X, fCentrePoint.Y+fOuterHalfAxes.Y,
          fCentrePoint.X + XLow, fCentrePoint.Y + YLow,
          fCentrePoint.X + Xzero, fCentrePoint.Y + Yzero);
    end;
    If (fHighZone > fLowZone) then begin
      Brush.Color := fOKZoneColor;
      Pie(fCentrePoint.X-fOuterHalfAxes.X, fCentrePoint.Y-fOuterHalfAxes.Y,
          fCentrePoint.X+fOuterHalfAxes.X, fCentrePoint.Y+fOuterHalfAxes.Y,
          fCentrePoint.X + XHi, fCentrePoint.Y + YHi,
          fCentrePoint.X + XLow, fCentrePoint.Y + YLow);
    end;
    If (fHighZone < fMax) then begin
      Brush.Color := fHighZoneColor;
      Pie(fCentrePoint.X-fOuterHalfAxes.X, fCentrePoint.Y-fOuterHalfAxes.Y,
          fCentrePoint.X+fOuterHalfAxes.X, fCentrePoint.Y+fOuterHalfAxes.Y,
          fCentrePoint.X - Xzero, fCentrePoint.Y + Yzero,
          fCentrePoint.X + XHi, fCentrePoint.Y + YHi);
    end;
  end;
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.ClearInnerFace;
Var Xzero, Yzero : LongInt;
Begin
  Xzero := XInEllipse(fZeroAngle,fOuterHalfAxes);
  Yzero := YInEllipse(fZeroAngle,fOuterHalfAxes);
  with fBitmap.Canvas do begin
    Pen.Width := 2;
    If Color = clBtnShadow then Pen.Color := cl3DLight
                           else Pen.Color := clBtnShadow;
    Arc(fCentrePoint.X-fInnerHalfAxes.X, fCentrePoint.Y-fInnerHalfAxes.Y,
        fCentrePoint.X+fInnerHalfAxes.X, fCentrePoint.Y+fInnerHalfAxes.Y,
        fCentrePoint.X - Xzero, fCentrePoint.Y + Yzero,
        fCentrePoint.X + Xzero, fCentrePoint.Y + Yzero);
    //overwrite colours:
    Pen.Color := Color;  //don't want edges of pie
    Brush.Color := Color;
    Ellipse(fCentrePoint.X-fInnerHalfAxes.X+2, fCentrePoint.Y-fInnerHalfAxes.Y+2,
        fCentrePoint.X+fInnerHalfAxes.X-2, fCentrePoint.Y+fInnerHalfAxes.Y-2);
  end;
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DrawTicks;
var i : LongInt;
    SmallEllipse : TPoint;
    Angle : Single;
Begin
  SmallEllipse.X := fInnerHalfAxes.X-((Width+Height) div 35);
  SmallEllipse.Y := fInnerHalfAxes.Y-((Width+Height) div 35);
  with fBitmap.Canvas do begin
    Pen.Color := fTickColor;
    Pen.Width := 1;
    For i := 1 to fTickCount do begin
      Angle := AngleOf(Min + (Max-Min)*(i-1)/(fTickCount-1));
      MoveTo(fCentrePoint.X + XInEllipse(Angle,fInnerHalfAxes),
             fCentrePoint.Y + YInEllipse(Angle,fInnerHalfAxes));
      LineTo(fCentrePoint.X + XInEllipse(Angle,SmallEllipse),
             fCentrePoint.Y + YInEllipse(Angle,SmallEllipse));
    end;
  end;
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DrawCaption;
var TW: LongInt; //textwidth
    H, H1, H2 : LongInt;
Begin
  H1 := fCentrePoint.Y - ((Width + Height) div 40); //top of knob
  H2 := fCentrePoint.Y - fInnerHalfAxes.Y - 1;      //bottom of scale
  H  := (H1 + H2) div 2;
  with fBitmap.Canvas do begin
    Brush.Color := Color; //background
    TW := TextWidth(fCaption) div 2;
    TextOut(fCentrePoint.X - TW, H, fCaption);
  end;
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DrawPointer;
var angle : Single;
    X,Y : LongInt;
    SmallEllipse : TPoint;
    Radius : LongInt;
Begin
  with fBitmap.Canvas do begin
    //hand:
    SmallEllipse.X := fInnerHalfAxes.X-3;
    SmallEllipse.Y := fInnerHalfAxes.Y-3;
    angle := AngleOf(fValue);
    X := XInEllipse(Angle,SmallEllipse);
    Y := YInEllipse(Angle,SmallEllipse);
    Pen.Width := 2;
    Pen.Color := fTickColor;
    MoveTo(fCentrePoint.X, fCentrePoint.Y);
    LineTo(fCentrePoint.X + X, fCentrePoint.Y + Y);
    //knob:
    Radius := (Width + Height) div 40;
    //shadow:
    Pen.Width := 3;
    Pen.Color := clBtnShadow;
    Ellipse(fCentrePoint.X - Radius+1, fCentrePoint.Y - Radius,
            fCentrePoint.X + Radius+1, fCentrePoint.Y + Radius);
    //knob itself, in colour of current zone:
    Pen.Width := 1;
    If (fLowZone > fMin) and (fValue <= fLowZone)
      then Brush.Color := fLowZoneColor
      else If (fHighZone < fMax) and (fValue >= fHighZone)
        then Brush.Color := fHighzoneColor
        else Brush.Color := fOKzoneColor;
    Pen.Color := Brush.Color;
    Ellipse(fCentrePoint.X - Radius, fCentrePoint.Y - Radius,
            fCentrePoint.X + Radius, fCentrePoint.Y + Radius);
  end;
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DrawValue;
var s : String;
Begin
  s := IntToStr(Value);
  with fBitmap.Canvas do begin
    Brush.Style := bsClear;
    TextOut(fCentrePoint.X - (TextWidth(s) div 2),
            Height- fFrameWidth -fTxtHeight -1, s);
  end;
End;
{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DeleteOldValue;
var s : String;
    c : TColor;
Begin
  s := IntToStr(fOldValue);
  with fBitmap.Canvas do begin
    Brush.Style := bsClear;
    c := Font.Color;
    Font.Color  := Color;
    TextOut(fCentrePoint.X - (TextWidth(s) div 2),
            Height- fFrameWidth -fTxtHeight -1, s);
    Font.Color := c; //reset
  end;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DrawOuterFace;
Begin
  ClearCanvas;
  DrawRegions;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.DrawInnerFace;
Begin
  ClearInnerFace;
  If fShowTicks and (TickCount > 1) then DrawTicks;
  If fCaption <> '' then DrawCaption;
  If fShowValue then DeleteOldValue;
  DrawPointer;
  If fShowValue then DrawValue;
  If fShowFrame then DrawFrame;
End;

{------------------------------------------------------------------------------}
Procedure TDFWAnalogMeter.Paint;
Begin
  BitBlt(Canvas.Handle, 0, 0, Width, Height,
         fBitmap.Canvas.Handle, 0, 0, SRCCOPY);
End;

function TDFWAnalogMeter.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWAnalogMeter.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWAnalogMeter.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWAnalogMeter.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
