{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWCompEditor.pas                                       }
{ Short description  :                                                         }
{ Author             : Carsten Butzek                                          }
{ E-Mail             : cabutzek@muenster.de                                    }
{ Internet           : http://www.delphifreeware.de                            }
{ Version            : 1.0                                                     }
{ Date last modified : 03/12/2000                                              }
{ Delphi versions    : 2,3,4,5                                                 }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        : Global About Component Editor                           }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       :                                                           }
{ Classes          : TDFWAboutComponentEditor, TDFWPageControlesEditor,        }
{                    TDFWAboutDialogEditor                                     }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWCompEditor;

interface
uses dsgnintf;
type
  TDFWAboutComponentEditor = class(TDefaultEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{$IFNDEF D2}
  TDFWPageControlesEditor = class(TDefaultEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;
 {$ENDIF}

  TDFWAboutDialogEditor = class(TDefaultEditor)
  public
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: integer; override;
    procedure ExecuteVerb(Index: integer); override;
  end;

implementation

uses DFWAbout, TypInfo, classes{$IFNDEF D2}, DFWPages{$ENDIF}, Sysutils, DFWAboutDialog;

 {Change Strings to your preferred language}
{$IFNDEF D2}
resourcestring
 sTABSHEET_DEFAULT_NAME = 'tbSheetes';
 sNEW_PAGE              = 'Neue Seite';
 sDEL_PAGE              = 'Seite lschen';
 sNEXT_PAGE             = 'Nchste Seite';
 sPREV_PAGE             = 'Vorherige Seite';
 {$ENDIF}

procedure TDFWAboutComponentEditor.ExecuteVerb(Index: Integer);
  function GetStringProperty(Instance: TPersistent; const PropName: string): string;
  var
    PropInfo: PPropInfo;
  begin
    Result := '';
    PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, PropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind in [tkString, tkLString{$IFNDEF D2}, tkWString{$ENDIF}]) then
      Result := GetStrProp(Instance, PropInfo);
  end;
begin
  case Index of
    0: ShowDFWAboutDialog(Component.ClassName,
                          GetStringProperty(Component, 'Version'),
                          GetStringProperty(Component, 'Author'),
                          GetStringProperty(Component, 'EMail'),
                          GetStringProperty(Component, 'Homepage'));
  else
    inherited ExecuteVerb(Index);
  end;
end;

function TDFWAboutComponentEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := 'About component';
  else
    Result := '';
  end;
end;

function TDFWAboutComponentEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;


{$IFNDEF D2}
procedure TDFWPageControlesEditor.ExecuteVerb(Index: Integer);
  function GetStringProperty(Instance: TPersistent; const PropName: string): string;
  var
    PropInfo: PPropInfo;
  begin
    Result := '';
    PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, PropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind in [tkString, tkLString, tkWString]) then
      Result := GetStrProp(Instance, PropInfo);
  end;
var
  NewPage: TDFWCustomTabSheetes;
  PControl : TDFWPageControles;
begin
  if Component is TDFWPageControles then
    PControl := TDFWPageControles(Component)
  else PControl := TDFWPageControles(TDFWTabSheetes(Component).PageControl);

  case Index of
    0: ShowDFWAboutDialog(Component.ClassName,
                          GetStringProperty(Component, 'Version'),
                          GetStringProperty(Component, 'Author'),
                          GetStringProperty(Component, 'EMail'),
                          GetStringProperty(Component, 'Homepage'));
    1:  begin  //  New Page
          NewPage := TDFWTabSheetes.Create(Designer.GetRoot);
          with NewPage do
          begin
            Parent      := PControl;
            PageControl := PControl;
            Caption     := sTABSHEET_DEFAULT_NAME + IntToStr(PControl.PageCount);
            Name        := Caption;
          end;
        end;
    2:  begin  //  Delete Page
          with PControl do
          begin
            NewPage := TDFWCustomTabSheetes(ActivePage);
            NewPage.PageControl := nil;
            NewPage.Free;
          end;
        end;
    3:  begin  //  Next Page
          PControl.FindNextPage(PControl.ActivePage,True,False);
        end;
    4:  begin  //  Previous Page
          PControl.FindNextPage(PControl.ActivePage,False,False);
        end;
  end;
  if Designer <> nil then Designer.Modified;
end;

function TDFWPageControlesEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0:  result := 'About component';
    1:  result := sNEW_PAGE;
    2:  result := sDEL_PAGE;
    3:  result := sNEXT_PAGE;
    4:  result := sPREV_PAGE;
  end;
end;

function TDFWPageControlesEditor.GetVerbCount: Integer;
begin
  result := 5;
end;

{$ENDIF}
function TDFWAboutDialogEditor.GetVerbCount: integer;
begin
  Result := 2;
end;

function TDFWAboutDialogEditor.GetVerb(Index: integer): string;
begin
  case index of
    0: Result := 'About component';
    1: Result := 'Test dialog';
  end;
end;

procedure TDFWAboutDialogEditor.ExecuteVerb(Index: Integer);
  function GetStringProperty(Instance: TPersistent; const PropName: string): string;
  var
    PropInfo: PPropInfo;
  begin
    Result := '';
    PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, PropName);
    if (PropInfo <> nil) and (PropInfo^.PropType^.Kind in [tkString, tkLString{$IFNDEF D2}, tkWString{$ENDIF}]) then
      Result := GetStrProp(Instance, PropInfo);
  end;
begin
  case index of
    0: ShowDFWAboutDialog(Component.ClassName,
                          GetStringProperty(Component, 'Version'),
                          GetStringProperty(Component, 'Author'),
                          GetStringProperty(Component, 'EMail'),
                          GetStringProperty(Component, 'Homepage'));
    1: (Component as TDFWAboutDialog).Execute;
  end;
end;

end.
