{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWPages.pas                                            }
{ Short description  :                                                         }
{ Author             : Erol S. Uzuner                                          }
{ E-Mail             : sarcon@tzi.de                                           }
{ Internet           :                                                         }
{ Version            : 1.0                                                     }
{ Date last modified : 09.09.99                                                }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     : Copyright (C) 1996-2000                                 }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWPageControles, TDFWPageControlBles                    }
{ Classes          : TDFWCustomPageControles, TDFWCustomTabSheetes,            }
{                    TDFWTabSheetes                                            }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWPages;

 { Attention!!!!
   Use Only True Type Fonts for TDFWPageControles if you use TabPosition
   tpLeft or tpRight. The TextRotating Method rotates only TrueTypeFonts
   if someone can tell me how to figure out true type and not true type fonts
   i will fix this limitation through drawing on bitmap rotating and copying on
   canvas for non True Type Fonts.
 }

interface
uses SysUtils, Forms, comctrls, Windows, Messages, Controls, Classes, Graphics, ExtCtrls,
     DFWGrafxes, DFWBitmapHolder, CommCtrl, DsgnIntf;

const
      ComponentVersion='1.0';
      ComponentAuthor='Erol S. Uzuner';
      ComponentEMail='sarcon@tzi.de';
      ComponentHomePage='';

type

  TDFWCustomPageControles = class;

  TDFWCustomTabSheetes = class(TTabSheet)
   private
    FCanvas: TCanvas;
    FTabColor: TColor;
    FTabFontColor: TColor;
    procedure SetTabColor(Value: TColor);
    procedure SetTabFontColor(Value: TColor);
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
  protected
    procedure Paint; virtual;
    procedure PaintWindow(DC: HDC); override;

    property Canvas: TCanvas read FCanvas;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property TabColor: TColor read FTabColor write SetTabColor;
    property TabFontColor: TColor read FTabFontColor write SetTabFontColor;
  end;


  TDFWTabSheetes = class(TDFWCustomTabSheetes)
  private
    { Private-Deklarationen }
    FBitmap : TBitmap;
    FDFWBitmapHolder : TDFWBitmapHolder;
    FImageChangeLink: TDFWLinkChange;

    procedure WMEraseBkgnd(var MSG: TWMEraseBkgnd); message wm_EraseBkgnd;  {Hintergund zeichnen}
    procedure DrawBackground(TheHandle : THandle);
  protected
    { Protected-Deklarationen }
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DrawBorder;
    procedure Paint; override;
    function GetBitmap : TBitmap;
    procedure SetBitmap(Value : TBitmap);
    function GetDFWBitmapHolder : TDFWBitmapHolder;
    procedure SetDFWBitmapHolder(Value : TDFWBitmapHolder);
    procedure DFWBitmapHolderChange(Sender: TObject);
    procedure SetBevelInner(Value: TBevelCut);
    procedure SetBevelOuter(Value: TBevelCut);
    procedure SetBevelWidth(Value: TBevelWidth);
    function getBevelInner : TBevelCut;
    function getBevelOuter : TBevelCut;
    function getBevelWidth : TBevelWidth;

  public
    { Public-Deklarationen }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property BevelInner: TBevelCut read getBevelInner write SetBevelInner default bvRaised;
    property BevelOuter: TBevelCut read getBevelOuter write SetBevelOuter default bvLowered;
    property BevelWidth : TBevelWidth read getBevelWidth write SetBevelWidth default 1;
    property Bitmap : TBitmap read GetBitmap write SetBitmap;
    property DFWBitmapHolder : TDFWBitmapHolder read GetDFWBitmapHolder write SetDFWBitmapHolder;
    property BorderWidth;
    property Color;
    property ParentColor;

  end;

  TDFWCustomPageControles = class(TPageControl)
  private
    FGradientFill : Boolean;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
  protected
    procedure Paint; virtual;
    procedure PaintWindow(DC: HDC); override;
    procedure DrawTab(TabIndex: Integer; const Rect: TRect; Active: Boolean); override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property TabFillGradient : Boolean read FGradientFill write FGradientFill stored true default false;
  end;

  TDFWPageControlBles=class(TPageControl)
  protected
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure WndProc(var Message:TMessage); override;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
  end;

 TDFWPageControles = class(TDFWCustomPageControles)
  private
    { Private-Deklarationen }
    FBitmap : TBitmap;
    FDFWBitmapHolder : TDFWBitmapHolder;
    FImageChangeLink: TDFWLinkChange;
    FNoBorder : Boolean;
    procedure WMEraseBkgnd(var MSG: TWMEraseBkgnd); message wm_EraseBkgnd;  {Hintergund zeichnen}
    procedure DrawBackground(TheHandle : THandle);

  protected
    { Protected-Deklarationen }
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
    procedure DrawBorder;
    function GetBitmap : TBitmap;
    procedure SetBitmap(Value : TBitmap);
    function GetDFWBitmapHolder : TDFWBitmapHolder;
    procedure SetDFWBitmapHolder(Value : TDFWBitmapHolder);
    procedure DFWBitmapHolderChange(Sender: TObject);
    procedure SetBevelInner(Value: TBevelCut);
    procedure SetBevelOuter(Value: TBevelCut);
    procedure SetBevelWidth(Value: TBevelWidth);
    function getBevelInner : TBevelCut;
    function getBevelOuter : TBevelCut;
    function getBevelWidth : TBevelWidth;
    procedure setNoBorder(value : Boolean);
    procedure WndProc(var Message:TMessage); override;
    procedure DrawTab(TabIndex: Integer; const Rect: TRect; Active: Boolean); override;

  public
    { Public-Deklarationen }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property Bitmap : TBitmap read GetBitmap write SetBitmap;
    property DFWBitmapHolder : TDFWBitmapHolder read GetDFWBitmapHolder write SetDFWBitmapHolder;
    property BevelInner: TBevelCut read getBevelInner write SetBevelInner default bvNone;
    property BevelOuter: TBevelCut read getBevelOuter write SetBevelOuter default bvNone;
    property BevelWidth : TBevelWidth read getBevelWidth write SetBevelWidth default 1;
    property BorderInvisible : Boolean read FNoBorder write setNoBorder default false;
  end;

implementation
uses ComStrs;

{ TDFWCustomTabSheetes }

constructor TDFWCustomTabSheetes.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCanvas := TControlCanvas.Create;
  TControlCanvas(FCanvas).Control := Self;
  TabColor := clBtnFace;
  TabFontColor := clBlack;
  Align := alClient;
  ControlStyle := ControlStyle + [csAcceptsControls, csNoDesignVisible];
  Visible := False;
end;

destructor TDFWCustomTabSheetes.Destroy;
begin
  FCanvas.Free;
  inherited Destroy;
end;

procedure TDFWCustomTabSheetes.WMPaint(var Message: TWMPaint);
begin
  ControlState := ControlState + [csCustomPaint];
  inherited;
  ControlState := ControlState - [csCustomPaint];
end;

procedure TDFWCustomTabSheetes.PaintWindow(DC: HDC);
begin
  FCanvas.Lock;
  try
    FCanvas.Handle := DC;
    try
      TControlCanvas(FCanvas).UpdateTextFlags;
      Paint;
    finally
      FCanvas.Handle := 0;
    end;
  finally
    FCanvas.Unlock;
  end;
end;

procedure TDFWCustomTabSheetes.Paint;
begin
end;

procedure TDFWCustomTabSheetes.SetTabColor(Value: TColor);
begin FTabColor := Value;
end;

procedure TDFWCustomTabSheetes.SetTabFontColor(Value: TColor);
begin
  FTabFontColor := Value;
end;

{TDFWTabSheetes}
constructor TDFWTabSheetes.Create(AOwner : TComponent);
begin
     inherited Create(AOwner);
     FBitmap := TBitmap.Create;
     FDFWBitmapHolder := NiL;
     FImageChangeLink := TDFWLinkChange.Create;
     FImageChangeLink.OnChange := DFWBitmapHolderChange;
end;

destructor TDFWTabSheetes.Destroy;
begin
     FBitmap.Free;
     if FDFWBitmapHolder <> NIL then
       FDFWBitmapHolder.UnRegisterChanges(FImageChangeLink);
     FImageChangeLink.Free;
     inherited Destroy;
end;

procedure TDFWTabSheetes.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Assigned(FDFWBitmapHolder) AND (Operation = opRemove) AND (AComponent = FDFWBitmapHolder) then
     FDFWBitmapHolder := NIL;
end;

procedure TDFWTabSheetes.WMEraseBkgnd(var MSG: TWMEraseBkgnd);
begin
  inherited;
  if (FBitmap.Empty) AND (NoT(Assigned(FDFWBitmapHolder)) OR
       (Assigned(FDFWBitmapHolder) AND (FDFWBitmapHolder.isEmpty))) then begin
       Canvas.Brush.Color := Color;
       Windows.FillRect(MSG.DC, GetClientRect, Canvas.Brush.Handle);
       DrawBorder;
  end else
    DrawBackground(MSG.DC);
end;

procedure TDFWTabSheetes.DrawBackground(TheHandle : THandle);
var Rect : TRect;
//    bmp : TBitmap;
begin
  Rect := GetClientRect;
  Rect.TopLeft := ClientRect.TopLeft;
  Rect.BottomRight := ClientRect.BottomRight;

  if NoT(FBitmap.Empty) then
      DrawTiled(TheHandle, rect, FBitmap)
  else if Assigned(FDFWBitmapHolder) AND (NoT FDFWBitmapHolder.isEmpty) then
    FDFWBitmapHolder.TiledOnH(rect, TheHandle);

 // if (Assigned(bmp)) then begin
  (*  if BevelInner <> bvNone then
      InflateRect(rect, -BevelWidth, -BevelWidth);
    if BevelOuter <> bvNone then
      InflateRect(rect, -BevelWidth, -BevelWidth);
  *)
//    DrawTiled(TheHandle, rect, bmp);
//  end;
  DrawBorder;
end;

procedure TDFWTabSheetes.DrawBorder;
var
  Rect: TRect;
  TopColor, BottomColor: TColor;

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;
begin
  Rect := GetClientRect;
  if NoT (BevelOuter In [bvNone, bvSpace]) then begin
    AdjustColors(BevelOuter);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;

  Frame3D(Canvas, Rect, Color, Color, BorderWidth);
  if BevelInner <> bvNone then begin
    AdjustColors(BevelInner);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;
end;

procedure TDFWTabSheetes.Paint;
var
  Rect: TRect;
//  FontHeight: Integer;

begin
  Rect := GetClientRect;
  with Canvas do begin
    Brush.Color := Color;
    if (FBitmap.Empty) AND (NoT(Assigned(FDFWBitmapHolder)) OR
       (Assigned(FDFWBitmapHolder) AND (FDFWBitmapHolder.isEmpty))) then begin
       FillRect(Rect);
       DrawBorder;
    end else
      DrawBackground(Canvas.handle);
  end;
end;

function TDFWTabSheetes.GetBitmap : TBitmap;
begin Result := FBitmap;
end;

procedure TDFWTabSheetes.SetBitmap(Value : TBitmap);
begin
   FBitmap.Assign(Value);
   if FDFWBitmapHolder <> nil then begin
    if Value <> NIL then
      FDFWBitmapHolder.UnRegisterChanges(FImageChangeLink)
    else
      FDFWBitmapHolder.RegisterChanges(FImageChangeLink);
   end;
   inherited Invalidate;
end;

function TDFWTabSheetes.GetDFWBitmapHolder : TDFWBitmapHolder;
begin Result := FDFWBitmapHolder;
end;

procedure TDFWTabSheetes.SetDFWBitmapHolder(Value : TDFWBitmapHolder);
begin
  if FDFWBitmapHolder <> nil then FDFWBitmapHolder.UnRegisterChanges(FImageChangeLink);
  FDFWBitmapHolder := Value;
  if FDFWBitmapHolder <> nil then
  begin
     FDFWBitmapHolder.RegisterChanges(FImageChangeLink);
     Value.FreeNotification(Self);
  end;
  invalidate;
end;

procedure TDFWTabSheetes.DFWBitmapHolderChange(Sender: TObject);
begin inherited invalidate;
end;

procedure TDFWTabSheetes.SetBevelInner(Value: TBevelCut);
begin
  inherited BevelInner := Value;
  Realign;
  Invalidate;
end;

procedure TDFWTabSheetes.SetBevelOuter(Value: TBevelCut);
begin
  inherited BevelOuter := Value;
  Realign;
  Invalidate;
end;

procedure TDFWTabSheetes.SetBevelWidth(Value: TBevelWidth);
begin
  inherited BevelWidth := Value;
  Realign;
  Invalidate;
end;

function TDFWTabSheetes.getBevelInner : TBevelCut;
begin Result := inherited BevelInner;
end;

function TDFWTabSheetes.getBevelOuter : TbevelCut;
begin Result := inherited BevelOuter;
end;

function TDFWTabSheetes.getBevelWidth : TBevelWidth;
begin Result := inherited BevelWidth;
end;


{ TDFWCustomPageControles }

constructor TDFWCustomPageControles.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OwnerDraw := True;
  ControlStyle := [csDoubleClicks, csOpaque];
  FGradientFill := false;
end;

destructor TDFWCustomPageControles.Destroy;
begin
  inherited Destroy;
end;

procedure TDFWCustomPageControles.WMPaint(var Message: TWMPaint);
begin
  //ControlState := ControlState + [csCustomPaint];
  inherited;
  //ControlState := ControlState - [csCustomPaint];
end;

procedure TDFWCustomPageControles.PaintWindow(DC: HDC);
begin
  inherited PaintWindow(DC);
  (*
  FCanvas.Lock;
  try
    FCanvas.Handle := DC;
    try
      TControlCanvas(FCanvas).UpdateTextFlags;
      Paint;
    finally
      FCanvas.Handle := 0;
    end;
  finally
    FCanvas.Unlock;
  end;
  *)
end;

procedure TDFWCustomPageControles.Paint;
begin
end;

procedure TDFWCustomPageControles.DrawTab(TabIndex: Integer; const Rect: TRect;
  Active: Boolean);

  procedure TextRotate(const S: string; x,y, deg : integer);
  var
    LogFont : tLogFont;
  begin
    GetObject(Self.Canvas.Font.handle, sizeof(tLogFont),@Logfont);
    
    Logfont.lfEscapement := deg * 10;
    Logfont.lfOrientation := Logfont.lfEscapement;
    Self.Canvas.font.handle := CreateFontIndirect(logfont);
    Self.Canvas.textout(x,y,S);
  end;

var R          : TRect;
    x, y, hr,
    wr, angle  : Integer;
    fs         : TSize;
    cap        : String;
begin
  inherited DrawTab(TabIndex, Rect, Active);
  with Canvas do
  begin
    Brush.Color := TDFWCustomTabSheetes(Pages[TabIndex]).TabColor;
    if FGradientFill then begin
      R := Rect;
      if Active then begin
        InflateRect(R, -1, 0);
        GradientFillRect(Canvas, R, Brush.Color,
                       TColor($00FFFFFF),
                       fdLeftToRight, R.Right-R.Left);
      end else begin
        case TabPosition of
          tpLeft  : begin
             InflateRect(R, 1, 0);
             OffsetRect(R, 1, 0);
            end;
          tpTop   : begin
             InflateRect(R, -1, 1);
             OffsetRect(R, 0, 1);
            end;
          tpRight  : begin
             InflateRect(R, 1, 0);
             OffsetRect(R, -1, 0);
            end;
          tpBottom : begin
             InflateRect(R, -1, 1);
             OffsetRect(R, 0, -1);
            end;
        end;

        GradientFillRect(Canvas, R, Brush.Color,
                       TColor($00888888),
                       fdLeftToRight, R.Right-R.Left);
      end;

    end else
      FillRect(Rect);

    Font.Color := TDFWCustomTabSheetes(Pages[TabIndex]).TabFontColor;
    Brush.Style := bsClear;
    R := Rect;

    cap := Pages[TabIndex].Caption;
    fs  := TextExtent(cap);

    hr  := R.Bottom - R.Top;
    wr  := R.Right  - R.Left;
    case TabPosition of
      tpTop, tpBottom : begin
          Inc(R.Left, (wr - fs.cx) shr 1);
          Inc(R.Top, (hr - fs.cy) shr 1);
        end;
      tpLeft, tpRight : begin
          Inc(R.Left, (wr - fs.cy) shr 1);
          Inc(R.Top, (hr - fs.cx) shr 1);
        end;
    end;

    case TabPosition of
      tpLeft, tpRight : begin
          SetBkMode(Canvas.Handle, Transparent);
          case TabPosition of
             tpLeft : begin
                 y := R.Top + fs.cx;
                 x := R.Left;
                 angle := 90;
               end;
             tpRight : begin
                 y := R.Top;
                 x := R.Left + fs.cy;
                 angle := 270;
               end;
          end;
          TextRotate(cap, x, y, angle);
       end;
      tpTop, tpBottom : TextRect(R,R.Left, R.Top, cap);
    end;
    //DrawFocusRect(Rect);
  end;
end;

procedure AdjustRect(var r : PRect; TabPosition : TTabPosition; tabsvisible : Boolean);
var d : Integer;
begin
    if tabsvisible then d := 4
    else d:= 4;

    if TabPosition = tpLeft then
         r^.Left:=r^.Left-d
    else r^.Left:=r^.Left-4;

    if TabPosition = tpRight then
          r^.Right:=r^.Right+d
    else  r^.Right:=r^.Right+4;

    if TabPosition = tpTop then
         r^.Top:=r^.Top-(d)
    else r^.Top:=r^.Top-6;

    if TabPosition = tpBottom then
         r^.Bottom:=r^.Bottom+d
    else r^.Bottom:=r^.Bottom+4;
end;

{TDFWPageControlBles}

procedure TDFWPageControlBles.WndProc(var Message:TMessage);
var i : integer;
    tabsvisible : Boolean;
begin
  if(Message.Msg=TCM_ADJUSTRECT) then begin
    Inherited WndProc(Message);
    tabsvisible := false;
    for i := 0 to PageCount-1 do begin
       if Pages[i].TabVisible then begin
           tabsvisible := true;
           break;
       end;
    end;
    AdjustRect(PRect(Message.LParam), TabPosition, tabsvisible);
  end else Inherited WndProc(Message);
end;


{ TDFWPageControles }
constructor TDFWPageControles.Create(AOwner : TComponent);
begin
     inherited Create(AOwner);
     FBitmap := TBitmap.Create;
     FDFWBitmapHolder := NiL;
     FImageChangeLink := TDFWLinkChange.Create;
     FImageChangeLink.OnChange := DFWBitmapHolderChange;
     BevelInner := bvNone;
     BevelOuter := bvNone;
     FNoBorder := false;
end;

destructor TDFWPageControles.Destroy;
begin
     FBitmap.Free;
     if FDFWBitmapHolder <> NIL then
       FDFWBitmapHolder.UnRegisterChanges(FImageChangeLink);
     FImageChangeLink.Free;

     inherited Destroy;
end;


procedure TDFWPageControles.WndProc(var Message:TMessage);
var i : integer;
    tabsvisible : Boolean;
begin
  if(Message.Msg=TCM_ADJUSTRECT) then begin
    Inherited WndProc(Message);
    if FNoBorder then begin
      tabsvisible := false;
      for i := 0 to PageCount-1 do begin
         if Pages[i].TabVisible then begin
             tabsvisible := true;
             break;
         end;
      end;
      AdjustRect(PRect(Message.LParam), TabPosition, tabsvisible);
    end;
  end else Inherited WndProc(Message);
end;

procedure TDFWPageControles.WMEraseBkgnd(var MSG: TWMEraseBkgnd);
begin
  inherited;
  if (FBitmap.Empty) AND (NoT(Assigned(FDFWBitmapHolder)) OR
       (Assigned(FDFWBitmapHolder) AND (FDFWBitmapHolder.Bitmap.Empty))) then begin
       Canvas.Brush.Color := Color;
       Windows.FillRect(MSG.DC, GetClientRect, Canvas.Brush.Handle);
       DrawBorder;
  end else
    DrawBackground(MSG.DC);
end;

procedure TDFWPageControles.DrawBackground(TheHandle : THandle);
var Rect : TRect;
    bmp : TBitmap;
begin
  Rect := GetClientRect;

  if NoT(FBitmap.Empty) then
    bmp := FBitmap
  else if Assigned(FDFWBitmapHolder) AND NoT(FDFWBitmapHolder.Bitmap.Empty) then
    bmp := FDFWBitmapHolder.Bitmap
  else bmp := NIL;

  if (Assigned(bmp)) then begin
    Rect.TopLeft := ClientRect.TopLeft;
    Rect.BottomRight := ClientRect.BottomRight;

    DrawTiled(TheHandle, rect, bmp);
  end;
  DrawBorder;
end;


procedure TDFWPageControles.DrawBorder;
var
  Rect: TRect;
  TopColor, BottomColor: TColor;

  procedure AdjustColors(Bevel: TBevelCut);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;
begin
  Rect := GetClientRect;
  if NoT (BevelOuter in [bvNone, bvSpace]) then begin
    AdjustColors(BevelOuter);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;

  Frame3D(Canvas, Rect, Color, Color, BorderWidth);
  if BevelInner <> bvNone then begin
    AdjustColors(BevelInner);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;
end;

procedure TDFWPageControles.Paint;
var
  Rect: TRect;
//  FontHeight: Integer;

begin
  Rect := GetClientRect;
  with Canvas do begin
    Brush.Color := Color;
    if (FBitmap.Empty) AND (NoT(Assigned(FDFWBitmapHolder)) OR
       (Assigned(FDFWBitmapHolder) AND (FDFWBitmapHolder.Bitmap.Empty))) then begin
       FillRect(Rect);
       DrawBorder;
    end else
      DrawBackground(Canvas.handle);
  end;
end;

procedure TDFWPageControles.DrawTab(TabIndex: Integer; const Rect: TRect; Active: Boolean);
//var r : TRect;
begin
  {  if NoT ((FBitmap.Empty) AND (NoT(Assigned(FDFWBitmapHolder)) OR
       (Assigned(FDFWBitmapHolder) AND (FDFWBitmapHolder.Bitmap.Empty)))) then begin

       r.Left := Rect.left+2; r.Top := rect.Top+2;
       r.Right := rect.Right-2; r.bottom := rect.bottom-2;
       DrawTiled(Canvas.Handle, r, DFWBitmapHolder.Bitmap);
    end; }
   inherited DrawTab(TabIndex, Rect, Active);
end;

function TDFWPageControles.GetBitmap : TBitmap;
begin Result := FBitmap;
end;

procedure TDFWPageControles.SetBitmap(Value : TBitmap);
begin
   FBitmap.Assign(Value);
   if FDFWBitmapHolder <> nil then begin
    if Value <> NIL then
      FDFWBitmapHolder.UnRegisterChanges(FImageChangeLink)
    else
      FDFWBitmapHolder.RegisterChanges(FImageChangeLink);
   end;
   inherited Invalidate;
end;

function TDFWPageControles.GetDFWBitmapHolder : TDFWBitmapHolder;
begin Result := FDFWBitmapHolder;
end;

procedure TDFWPageControles.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Assigned(FDFWBitmapHolder) AND (Operation = opRemove) AND (AComponent = FDFWBitmapHolder) then
     FDFWBitmapHolder := NIL;
end;

procedure TDFWPageControles.SetDFWBitmapHolder(Value : TDFWBitmapHolder);
begin
  if FDFWBitmapHolder <> nil then FDFWBitmapHolder.UnRegisterChanges(FImageChangeLink);
  FDFWBitmapHolder := Value;
  if FDFWBitmapHolder <> nil then
  begin
     FDFWBitmapHolder.RegisterChanges(FImageChangeLink);
     Value.FreeNotification(Self);
  end;
  invalidate;
end;

procedure TDFWPageControles.DFWBitmapHolderChange(Sender: TObject);
begin inherited invalidate;
end;

procedure TDFWPageControles.SetBevelInner(Value: TBevelCut);
begin
  inherited BevelInner := Value;
  Realign;
  Invalidate;
end;

procedure TDFWPageControles.SetBevelOuter(Value: TBevelCut);
begin
  inherited BevelOuter := Value;
  Realign;
  Invalidate;
end;

procedure TDFWPageControles.SetBevelWidth(Value: TBevelWidth);
begin
  inherited BevelWidth := Value;
  Realign;
  Invalidate;
end;

function TDFWPageControles.getBevelInner : TBevelCut;
begin Result := inherited BevelInner;
end;

function TDFWPageControles.getBevelOuter : TbevelCut;
begin Result := inherited BevelOuter;
end;

function TDFWPageControles.getBevelWidth : TBevelWidth;
begin Result := inherited BevelWidth;
end;

procedure TDFWPageControles.setNoBorder(value : Boolean);
begin
 if FNoBorder <> Value then begin
   FNoBorder := Value;
   Realign;
   Invalidate;
 end;
end;

function TDFWPageControles.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWPageControles.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWPageControles.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWPageControles.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

function TDFWPageControlBles.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWPageControlBles.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWPageControlBles.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWPageControlBles.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.

