{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWSTFonts.pas                                          }
{ Short description  :                                                         }
{ Author             : Jakub Dusek                                             }
{ E-Mail             : j.dusek@seznam.cz                                       }
{ Internet           : http://dusek.zde.cz                                     }
{ Version            : 1.21                                                    }
{ Date last modified : 10.12.1999                                              }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWSTFontsCombo, TDFWSTFontsList                         }
{ Classes          : TSTHintWindow, TPreview                                   }
{ Types            : TFontType, TFontsType, TPreviewPosition                   }
{ Constants        : TestSentence                                              }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWSTFonts;

{$IFNDEF VER80}
 {$IFNDEF VER90}
  {$IFNDEF VER93}
    {$IFNDEF VER100}
      {$IFNDEF VER110}
        {$DEFINE ST4}
        {$IFNDEF VER120}
           {$IFNDEF VER125}
            {$DEFINE ST5}
          {$ENDIF}
        {$ENDIF}
      {$ENDIF}
    {$ENDIF}
  {$ENDIF}
 {$ENDIF}
{$ENDIF}

interface

uses
  Windows, Messages, Controls, Classes, Graphics,
  StdCtrls, SysUtils, Printers, Forms;

const
  TestSentence = 'The quick brown fox jumps over the lazy dog';

  ComponentVersion='1.21';
  ComponentAuthor='Jakub Dusek';
  ComponentEMail='j.dusek@seznam.cz';
  ComponentHomePage='http://dusek.zde.cz';

type
  TFontType = (TrueTypeFonts, RasterFonts, DeviceFonts, OtherFonts);

  TFontsType = set of TFontType;

  TPreviewPosition = (ppLeftTop, ppLeftBottom, ppRightTop, ppRightBottom);

  TSTHintWindow = class (THintWindow)
  protected
    procedure Paint; override;
  end;
  
  TPreview = class (TPersistent)
  private
    FColor		: TColor;
    FFontColor		: TColor;
    FFontSize		: Integer;
    FFontStyle		: TFontStyles;
    FMaxWidth		: Integer;
    FMargin		: Integer;
    FPosition		: TPreviewPosition;
    FText		: string;
    FVisible		: Boolean;
  published
    property Color	: TColor read FColor write FColor default clInfoBk;
    property FontColor	: TColor read FFontColor write FFontColor default clInfoText;
    property FontSize	: Integer read FFontSize write FFontSize default 14;
    property FontStyle	: TFontStyles read FFontStyle write FFontStyle;
    property MaxWidth	: Integer read FMaxWidth write FMaxWidth default 200;
    property Margin	: Integer read FMargin write FMargin default 2;
    property Position	: TPreviewPosition read FPosition write FPosition default ppRightTop;
    property Text	: string read FText write FText;
    property Visible	: Boolean read FVisible write FVisible default True;
  end;

  TDFWSTFontsCombo = class (TCustomComboBox)
  private
    FAutoUpdate		: Boolean;
    FTrueTypeBmp	: TBitmap;
    FRasterBmp		: TBitmap;
    FDeviceBmp		: TBitmap;
    FFontName		: TFontName;
    FFontsType		: TFontsType;
    FNoFonts		: TStringList;
    FPreview		: TPreview;
    FHintWindow		: TSTHintWindow;
    FBitmap		: TBitmap;
    FBmpWidth		: Integer;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure SetFontName (Value: TFontName);
    procedure SetFontsType (Value: TFontsType);
    procedure SetNoFonts (Value: TStringList);
    procedure UpdateHeight;
    procedure NoFontsChanged (Sender: TObject);
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure CNCommand (var Message: TWMCommand);  message CN_COMMAND;
  protected
    procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Change; override;
  public
    constructor Create (AOwner: TComponent); override;
    procedure CreateWnd; override;
    procedure Loaded; override;
    destructor Destroy; override;
    procedure SetTrueTypeBmp (Value: TBitmap);
    procedure SetRasterBmp (Value: TBitmap);
    procedure SetDeviceBmp (Value: TBitmap);
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property AutoUpdate		: Boolean read FAutoUpdate write FAutoUpdate default True;
    property FontName		: TFontName read FFontName write SetFontName;
    property FontsType		: TFontsType read FFontsType write SetFontsType default [TrueTypeFonts, RasterFonts, DeviceFonts, OtherFonts];
    property TrueTypeBmp	: TBitmap read FTrueTypeBmp write SetTrueTypeBmp;
    property RasterBmp		: TBitmap read FRasterBmp write SetRasterBmp;
    property DeviceBmp		: TBitmap read FDeviceBmp write SetDeviceBmp;
    property NoFonts		: TStringList read FNoFonts write SetNoFonts;
    property Preview		: TPreview read FPreview write FPreview;
    procedure UpdateFonts;
    {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    {$ENDIF}
    {$IFDEF ST5}
    property OnConTextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

  TDFWSTFontsList = class (TCustomListBox)
  private
    FTrueTypeBmp	: TBitmap;
    FRasterBmp		: TBitmap;
    FDeviceBmp		: TBitmap;
    FFontName		: TFontName;
    FFontsType		: TFontsType;
    FNoFonts		: TStringList;
    FPreview		: TPreview;
    FHintWindow		: TSTHintWindow;
    FBitmap		: TBitmap;
    FBmpWidth		: Integer;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure SetFontName (Value: TFontName);
    procedure SetFontsType (Value: TFontsType);
    procedure SetNoFonts (Value: TStringList);
    procedure UpdateHeight;
    procedure NoFontsChanged (Sender: TObject);
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
  protected
    procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Click; override;
  public
    constructor Create (AOwner: TComponent); override;
    procedure CreateWnd; override;
    procedure Loaded; override;
    destructor Destroy; override;
    procedure SetTrueTypeBmp (Value: TBitmap);
    procedure SetRasterBmp (Value: TBitmap);
    procedure SetDeviceBmp (Value: TBitmap);
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
  published
    property FontName		: TFontName read FFontName write SetFontName;
    property FontsType		: TFontsType read FFontsType write SetFontsType default [TrueTypeFonts, RasterFonts, DeviceFonts, OtherFonts];
    property TrueTypeBmp	: TBitmap read FTrueTypeBmp write SetTrueTypeBmp;
    property RasterBmp		: TBitmap read FRasterBmp write SetRasterBmp;
    property DeviceBmp		: TBitmap read FDeviceBmp write SetDeviceBmp;
    property NoFonts		: TStringList read FNoFonts write SetNoFonts;
    property Preview		: TPreview read FPreview write FPreview;
    procedure UpdateFonts;
    {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    {$ENDIF}
    {$IFDEF ST5}
    property OnContextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property Align;
    property BorderStyle;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

{$R DFWSTFonts.RES}

implementation

function GetItemHeight (Font: TFont): Integer;
var
  DC		: HDC;
  Fnt		: HFont;
  Metrics	: TTextMetric;
begin
  DC := GetDC (0);
  try
    Fnt := SelectObject (DC, Font.Handle);
    GeTTextMetrics (DC, Metrics);
    SelectObject (DC, Fnt);
  finally
    ReleaseDC (0, DC);
  end;
  Result := Metrics.tmHeight + 1;
end;

procedure TSTHintWindow.Paint;
var
  R: TRect;
begin
  R := ClientRect;
  Inc (R.Left, 2);
  Inc (R.Top, 2);
  DrawText (Canvas.Handle, PChar(Caption), -1, R, DT_Left or DT_NOPREFIX or
            DT_WORDBREAK or DrawTextBiDiModeFlagsReadingOnly);
end;

constructor TDFWSTFontsCombo.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FHintWindow := TSTHintWindow.Create (Self);
  FHintWindow.Canvas.Font.Charset := DEFAULT_CHARSET;
  FTrueTypeBmp := TBitmap.Create;
  FRasterBmp := TBitmap.Create;
  FDeviceBmp := TBitmap.Create;
  FBitmap := TBitmap.Create;
  FTrueTypeBmp.Handle := LoadBitmap (HInstance, 'STTTF');
  FRasterBmp.Handle := LoadBitmap (HInstance, 'STFON');
  FDeviceBmp.Handle := LoadBitmap (HInstance, 'STPRN');
  FNoFonts := TStringList.Create;
  TStringList(FNoFonts).OnChange := NoFontsChanged;
  Style := csOwnerDrawFixed;
  Sorted := True;
  FPreview := TPreview.Create;
  FPreview.FVisible := True;
  FPreview.FColor := clInfoBk;
  FPreview.FFontColor := clInfoText;
  FPreview.FPosition := ppRightTop;
  FPreview.FText := TestSentence;
  FPreview.FFontSize := 14;
  FPreview.FMaxWidth := 200;
  FPreview.FMargin := 2;
  FFontsType := [TrueTypeFonts, RasterFonts, DeviceFonts,OtherFonts];
  FAutoUpdate := True;
  UpdateHeight;
end;

procedure TDFWSTFontsCombo.CreateWnd;
begin
  inherited CreateWnd;
  UpdateFonts;
end;

procedure TDFWSTFontsCombo.Loaded;
begin
  inherited Loaded;
  SetFontName (FFontName);
end;

destructor TDFWSTFontsCombo.Destroy;
begin
  FTrueTypeBmp.Free;
  FRasterBmp.Free;
  FDeviceBmp.Free;
  FNoFonts.Free;
  FBitmap.Free;
  FHintWindow.Free;
  FPreview.Free;
  inherited Destroy;
end;

procedure TDFWSTFontsCombo.NoFontsChanged (Sender: TObject);
begin
  UpdateFonts;
end;

procedure TDFWSTFontsCombo.SetNoFonts (Value: TStringList);
begin
  FNoFonts.Assign (Value);
  UpdateFonts;
end;

procedure TDFWSTFontsCombo.SetTrueTypeBmp (Value: TBitmap);
begin
  FTrueTypeBmp.Assign (Value);
  UpdateHeight;
end;

procedure TDFWSTFontsCombo.SetRasterBmp (Value: TBitmap);
begin
  FRasterBmp.Assign (Value);
  UpdateHeight;
end;

procedure TDFWSTFontsCombo.SetDeviceBmp (Value: TBitmap);
begin
  FDeviceBmp.Assign (Value);
  UpdateHeight;
end;

procedure TDFWSTFontsCombo.UpdateHeight;
var i: Integer;
begin
  i := GetItemHeight (Font);
  if FTrueTypeBmp.Height > i then i := FTrueTypeBmp.Height;
  if FRasterBmp.Height > i then i := FRasterBmp.Height;
  if FDeviceBmp.Height > i then i := FDeviceBmp.Height;
  ItemHeight := i + 2;
  Height := ItemHeight;
  FBmpWidth := FTrueTypeBmp.Width;
  if FRasterBmp.Width > FBmpWidth then FBmpWidth := FRasterBmp.Width;
  if FDeviceBmp.Width > FBmpWidth then FBmpWidth := FDeviceBmp.Width;
  RecreateWnd;
end;

procedure TDFWSTFontsCombo.CMFontChanged (var Message: TMessage);
begin
  inherited;
  UpdateHeight;
end;

procedure TDFWSTFontsCombo.CNCommand (var Message: TWMCommand);
begin
  inherited;
  case Message.NotifyCode of
    CBN_DropDown:
    begin
      if FAutoUpdate then UpdateFonts;
    end;
  end;
end;

procedure TDFWSTFontsCombo.Change;
begin
  FFontName := Items [ItemIndex];
  inherited Change;
end;

procedure TDFWSTFontsCombo.SetFontName (Value: TFontName);
var Index: integer;
begin
  Index := Items.IndexOf (Value);   
  if Index <> - 1 then
  begin
    ItemIndex := Index;
    FFontName := Value;
  end;
  if Assigned (OnChange) then OnChange (Self);
end;

procedure TDFWSTFontsCombo.SetFontsType(Value:TFontsType);
begin
  FFontsType := Value;
  UpdateFonts;
end;

procedure TDFWSTFontsCombo.UpdateFonts;

function IsValid (Combo: TDFWSTFontsCombo; FontType: Integer): Boolean;
begin
  if (FontType and TrueTYPE_FONTTYPE) <> 0 then
  begin
    Result := TrueTypeFonts in Combo.FontsType;
  end
  else
    if (FontType and DEVICE_FONTTYPE) <> 0 then
    begin
      Result := DeviceFonts in Combo.FontsType;
    end
    else
      if (FontType and RASTER_FONTTYPE) <> 0 then
      begin
        Result := RasterFonts in Combo.FontsType;
      end
      else
        Result := OtherFonts in Combo.FontsType;
end;

function EnumFontsProc (var EnumLogFont: TEnumLogFont; var TextMetric: TNewTextMetric; 
                        FontType: Integer; Data: LPARAM): Integer;
  export; stdcall;
var
  FaceName: string;
begin
  FaceName := StrPas (EnumLogFont.elfLogFont.lfFaceName);
  with TDFWSTFontsCombo (Data) do
    if (Items.IndexOf (FaceName) = - 1)
        and (IsValid (TDFWSTFontsCombo (Data), FontType))
             and (NoFonts.IndexOf (FaceName) = - 1) then
    begin
      Items.AddObject (FaceName, TObject (FontType));
    end;
  Result := 1;
end;

var
  DC	: HDC;
  Old	: string;
  Index	: integer;
begin
  if not HandleAllocated then Exit;
  if Items.Count <> 0 then Old := Items [ItemIndex];
  Items.BeginUpdate;
  try
    Clear;
    DC := GetDC (0);
    try
      EnumFontFamilies (DC, nil, @EnumFontsProc, Longint (Self));
      try
        EnumFontFamilies (Printer.Handle, nil, @EnumFontsProc, Longint (Self));
      except
      end;
    finally
      ReleaseDC (0, DC);
    end;
  finally
    Items.EndUpdate;
  end;
  Index := Items.IndexOf (Old);
  if Index <> - 1 then ItemIndex := Index
  else
  begin
    if Items.Count <> 0 then
    begin
      ItemIndex := 0;
      SetFontName ( Items [ItemIndex]);
    end;
  end;
  if Items.Count <> 0 then SetFontName (Items [ItemIndex]);
end;

procedure TDFWSTFontsCombo.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
var FontType			: Integer;
    HintRect,ItemRect, 
    BitmapRect			: TRect;
    Pos				: TPoint;
    s				: string;
begin
  if (DroppedDown) and (FPreview.FVisible) then
  begin
    s := FPreview.FText;
    if s = '%1' then s := Items [ItemIndex];
    FHintWindow.Canvas.Font.Name := Items [ItemIndex];
    FHintWindow.Canvas.Font.Size := FPreview.FFontSize;
    FHintWindow.Canvas.Font.Color := FPreview.FFontColor;
    FHintWindow.Canvas.Font.Style := FPreview.FFontStyle;
    FHintWindow.Color := FPreview.FColor;
    HintRect := FHintWindow.CalcHintRect (FPreview.FMaxWidth, s, nil);
    case FPreview.Position of
      ppLeftTop: Pos := ClientToScreen (Point (0 - (HintRect.Right - HintRect.Left) - FPreview.FMargin, 0));
      ppLeftBottom: Pos := ClientToScreen (Point (0 - (HintRect.Right - HintRect.Left) - FPreview.FMargin,Height));
      ppRightTop: Pos := ClientToScreen (Point (Width + FPreview.FMargin, 0));
      ppRightBottom: Pos := ClientToScreen (Point (Width + FPreview. FMargin, Height));
    end;
    OffsetRect (HintRect, Pos.X, Pos.Y);
    FHintWindow.ActivateHint (HintRect, s);
  end
  else
  begin
    FHintWindow.ReleaseHandle;
  end;
  FBitmap.Assign (nil);
  Canvas.FillRect (Rect);
  FontType := Integer (Items.Objects [Index]);
  if (FontType and TrueTYPE_FONTTYPE) <> 0 then FBitmap.Assign (FTrueTypeBmp)
  else
    if (FontType and DEVICE_FONTTYPE) <> 0 then FBitmap.Assign (FDeviceBmp)
    else
      if (FontType and RASTER_FONTTYPE) <> 0 then FBitmap.Assign (FRasterBmp);
  ItemRect := Bounds (Rect.Left + 2, (Rect.Top + Rect.Bottom - FBitmap.Height) div 2, FBitmap.Width, FBitmap.Height);
  BitmapRect := Bounds (0, 0, FBitmap.Width, FBitmap.Height);
  Canvas.BrushCopy (ItemRect, FBitmap, BitmapRect, FBitmap.TransParentColor);
  Rect.Left := Rect.Left + FBmpWidth + 4;
  DrawText (Canvas.Handle, PChar (Items [Index]), Length (Items [Index]),
           Rect, DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
end;

constructor TDFWSTFontsList.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FHintWindow := TSTHintWindow.Create (Self);
  FHintWindow.Canvas.Font.Charset := DEFAULT_CHARSET;
  FTrueTypeBmp := TBitmap.Create;
  FRasterBmp := TBitmap.Create;
  FDeviceBmp := TBitmap.Create;
  FBitmap := TBitmap.Create;
  FTrueTypeBmp.Handle := LoadBitmap (HInstance, 'STTTF');
  FRasterBmp.Handle := LoadBitmap (HInstance, 'STFON');
  FDeviceBmp.Handle := LoadBitmap (HInstance, 'STPRN');
  FNoFonts := TStringList.Create;
  TStringList (FNoFonts).OnChange := NoFontsChanged;
  Style := lbOwnerDrawFixed;
  Sorted := True;
  FPreview := TPreview.Create;
  FPreview.FVisible := True;
  FPreview.FColor := clInfoBk;
  FPreview.FFontColor := clInfoText;
  FPreview.FPosition := ppRightTop;
  FPreview.FText := TestSentence;
  FPreview.FFontSize := 14;
  FPreview.FMaxWidth := 200;
  FPreview.FMargin := 2;
  FFontsType := [TrueTypeFonts, RasterFonts, DeviceFonts,OtherFonts];
  UpdateHeight;
end;

procedure TDFWSTFontsList.CreateWnd;
begin
  inherited CreateWnd;
  UpdateFonts;
end;

procedure TDFWSTFontsList.Loaded;
begin
  inherited Loaded;
  SetFontName (FFontName);
end;

destructor TDFWSTFontsList.Destroy;
begin
  FTrueTypeBmp.Free;
  FRasterBmp.Free;
  FDeviceBmp.Free;
  FNoFonts.Free;
  FBitmap.Free;
  FHintWindow.Free;
  FPreview.Free;
  inherited Destroy;
end;

procedure TDFWSTFontsList.NoFontsChanged (Sender: TObject);
begin
  UpdateFonts;
end;

procedure TDFWSTFontsList.SetNoFonts (Value: TStringList);
begin
  FNoFonts.Assign (Value);
  UpdateFonts;
end;

procedure TDFWSTFontsList.SetTrueTypeBmp (Value: TBitmap);
begin
  FTrueTypeBmp.Assign(Value);
  UpdateHeight;
end;

procedure TDFWSTFontsList.SetRasterBmp (Value: TBitmap);
begin
  FRasterBmp.Assign (Value);
  UpdateHeight;
end;

procedure TDFWSTFontsList.SetDeviceBmp (Value: TBitmap);
begin
  FDeviceBmp.Assign (Value);
  UpdateHeight;
end;

procedure TDFWSTFontsList.UpdateHeight;
var i: Integer;
begin
  i := GetItemHeight (Font);
  if FTrueTypeBmp.Height > i then i := FTrueTypeBmp.Height;
  if FRasterBmp.Height > i then i := FRasterBmp.Height;
  if FDeviceBmp.Height > i then i := FDeviceBmp.Height;
  ItemHeight := i + 2;
  FBmpWidth := FTrueTypeBmp.Width;
  if FRasterBmp.Width > FBmpWidth then FBmpWidth := FRasterBmp.Width;
  if FDeviceBmp.Width > FBmpWidth then FBmpWidth := FDeviceBmp.Width;
  RecreateWnd;
end;

procedure TDFWSTFontsList.CMFontChanged (var Message: TMessage);
begin
  inherited;
  UpdateHeight;
end;

procedure TDFWSTFontsList.Click;
begin
  FFontName := Items [ItemIndex];
  inherited Click;
end;

procedure TDFWSTFontsList.SetFontName (Value: TFontName);
var Index: integer;
begin
  Index := Items.IndexOf (Value);   
  if Index <> - 1 then
  begin
    ItemIndex := Index;
    FFontName := Value;
  end;
  if Assigned (OnClick) then OnClick (Self);
end;

procedure TDFWSTFontsList.SetFontsType (Value: TFontsType);
begin
  FFontsType := Value;
  UpdateFonts;
end;

procedure TDFWSTFontsList.UpdateFonts;

function IsValid (Combo: TDFWSTFontsList;FontType:Integer): Boolean;
begin
  if (FontType and TrueTYPE_FONTTYPE) <> 0 then
  begin
    Result := TrueTypeFonts in Combo.FontsType;
  end
  else
    if (FontType and DEVICE_FONTTYPE) <> 0 then
    begin
      Result := DeviceFonts in Combo.FontsType;
    end
    else
      if (FontType and RASTER_FONTTYPE) <> 0 then
      begin
        Result := RasterFonts in Combo.FontsType;
      end
      else
        Result := OtherFonts in Combo.FontsType;
end;

function EnumFontsProc (var EnumLogFont: TEnumLogFont; var TextMetric: TNewTextMetric; 
                        FontType: Integer; Data: LPARAM): Integer;
  export; stdcall;
var
  FaceName: string;
begin
  FaceName := StrPas (EnumLogFont.elfLogFont.lfFaceName);
  with TDFWSTFontsCombo (Data) do
    if (Items.IndexOf (FaceName) = - 1)
        and (IsValid (TDFWSTFontsList (Data), FontType))
             and (NoFonts.IndexOf (FaceName) = - 1) then
    begin
      Items.AddObject (FaceName, TObject (FontType));
    end;
  Result := 1;
end;

var
  DC	: HDC;
  Old	: string;
  Index	: integer;
begin
  if not HandleAllocated then Exit;
  if Items.Count <> 0 then Old := Items [ItemIndex];
  Items.BeginUpdate;
  try
    Clear;
    DC := GetDC (0);
    try
      EnumFontFamilies (DC, nil, @EnumFontsProc, Longint (Self));
      try
        EnumFontFamilies (Printer.Handle, nil, @EnumFontsProc, Longint (Self));
      except
      end;
    finally
      ReleaseDC (0, DC);
    end;
  finally
    Items.EndUpdate;
  end;
  Index := Items.IndexOf (Old);
  if Index <> - 1 then ItemIndex := Index
  else
  begin
    if Items.Count <> 0 then
    begin
      ItemIndex := 0;
      SetFontName ( Items [ItemIndex]);
    end;
  end;
  if Items.Count <> 0 then SetFontName (Items [ItemIndex]);
end;

procedure TDFWSTFontsList.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
var FontType, H, W	: Integer;
    HintRect, ItemRect,
    BitmapRect		: TRect;
    Pos			: TPoint;
    s			: string;
begin
  if (Focused) and (FPreview.FVisible) then
  begin
    s := FPreview.FText;
    if s = '%1' then s := Items [ItemIndex];
    FHintWindow.Canvas.Font.Name := Items [ItemIndex];
    FHintWindow.Canvas.Font.Size := FPreview.FFontSize;
    FHintWindow.Canvas.Font.Color := FPreview.FFontColor;
    FHintWindow.Canvas.Font.Style := FPreview.FFontStyle;
    FHintWindow.Color := FPreview.FColor;
    HintRect := FHintWindow.CalcHintRect (FPreview.FMaxWidth, s, nil);
    if BorderStyle = bsNone then
    begin
      W := 0;
      H := 0;
    end
    else
    begin
      W := 2;
      H := 2;
    end;
    case FPreview.Position of
      ppLeftTop: Pos := ClientToScreen (Point (0 - (HintRect.Right - HintRect.Left) - FPreview.FMargin - w,0 - h));
      ppLeftBottom: Pos := ClientToScreen (Point (0 - (HintRect.Right - HintRect.Left) - FPreview.FMargin - w,Height - (HintRect.Bottom - HintRect.Top) - H - 4));
      ppRightTop: Pos := ClientToScreen (Point (Width + FPreview.FMargin - w,0 - h));
      ppRightBottom: Pos := ClientToScreen (Point (Width + FPreview.FMargin - w, Height - (HintRect.Bottom - HintRect.Top) - H - 4));
    end;
    OffsetRect (HintRect, Pos.X, Pos.Y);
    FHintWindow.ActivateHint (HintRect, s);
  end
  else
  begin
    FHintWindow.ReleaseHandle;
  end;
  FBitmap.Assign (nil);
  Canvas.FillRect (rect);
  FontType := Integer (Items.Objects [Index]);
  if (FontType and TRUETYPE_FONTTYPE) <> 0 then FBitmap.Assign(FTrueTypeBmp)
  else
    if (FontType and DEVICE_FONTTYPE) <> 0 then FBitmap.Assign(FDeviceBmp)
    else
      if (FontType and RASTER_FONTTYPE) <> 0 then FBitmap.Assign(FRasterBmp);
  ItemRect := Bounds (Rect.Left + 2, (Rect.Top + Rect.Bottom - FBitmap.Height) div 2, FBitmap.Width, FBitmap.Height);
  BitmapRect := Bounds (0, 0, FBitmap.Width, FBitmap.Height);
  Canvas.BrushCopy (ItemRect, FBitmap, BitmapRect, FBitmap.TransParentColor);
  Rect.Left := Rect.Left + FBmpWidth + 4;
  DrawText (Canvas.Handle, PChar (Items [Index]), Length (Items [Index]),
           Rect, DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
end;

function TDFWSTFontsCombo.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWSTFontsCombo.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWSTFontsCombo.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWSTFontsCombo.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

function TDFWSTFontsList.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWSTFontsList.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWSTFontsList.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWSTFontsList.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
